module RolifyHier
	class HierRole
		@@roles = {}.with_indifferent_access

		def self.[](name)
			if @@roles.key? name
				@@roles[name]
			else
				role = self.new(name)
				@@roles[name] = role
				role
			end
		end

		attr_reader :name
		attr_reader :sub

		def initialize(name)
			raise ArgumentError, 'name is nil' if name.nil?
			@name = name
			@sub = Set.new
		end

		def <<(o)
			check_arg_type(o)
			include(o)
		end

		def include(o)
			check_arg_type(o)
			@sub << o
			o
		end

		def include?(o)
			check_arg_type(o)
			@sub.include?(o)
		end

		# Not hierarchic
		def remove(o)
			check_arg_type(o)
			@sub.remove(o)
		end

		def reachable_roles
			list = [self]
			@sub.each do |e|
				list += e.reachable_roles
			end
			list
		end

		private
			def check_arg_type(o)
				raise 'the argument is not an HierRole class instance' unless o.instance_of?(HierRole)
			end
	end
end